package com.bootdo.datas.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.bootdo.common.config.BootdoConfig;
import com.bootdo.common.utils.FileUtil;
import com.bootdo.common.utils.ShiroUtils;
import com.bootdo.common.utils.StringUtils;
import com.bootdo.datas.dao.GyDataDao;
import com.bootdo.datas.dao.GyUnitDao;
import com.bootdo.datas.domain.*;
import com.bootdo.datas.dto.ExamineLogDTO;
import com.bootdo.datas.dto.GyDataImportDTO;
import com.bootdo.datas.service.ExamineLogService;
import com.bootdo.datas.service.GyDataService;
import com.bootdo.datas.service.UnitService;
import com.bootdo.datas.tools.ExcelUtils;
import com.bootdo.datas.tools.gm.FileCryptTool;
import com.bootdo.system.domain.RoleDO;
import com.bootdo.system.domain.UserDO;
import com.bootdo.system.service.RoleService;
import com.google.common.collect.Lists;
import org.apache.commons.lang.ObjectUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional
public class GyDataServiceImpl implements GyDataService {

    @Autowired
    private BootdoConfig bootdoConfig;
    @Autowired
    private GyUnitDao gyUnitDao;
    @Autowired
    private GyDataDao gyDataDao;
    @Value("${managerValue}")
    public String managerValue;
    @Autowired
    private UnitService unitService;
    @Autowired
    private ExamineLogService examineLogService;
    @Autowired
    private RoleService roleService;

    private static Logger log = LoggerFactory.getLogger(GyDataServiceImpl.class);

    @Override
    public List<GyDataImportDTO> list(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        log.info("managerValue:" + managerValue);
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("unitCity", String.valueOf(currUser.getCity()));
        } else {
            log.info("工信部 管理员");
        }
        return gyDataDao.list(map);
    }

    @Override
    public List<GyDataImportDTO> examineList(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        log.info("managerValue:" + managerValue);
        //
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("unitCity", String.valueOf(currUser.getCity()));
        } else {
            log.info("工信部 管理员");
        }
        if (!managerValue.contains(currUser.getDeptId().toString()) && currUser.getExamineRoleId() != null) {
            map.put("roleId", String.valueOf(currUser.getExamineRoleId()));
        }

        if (map.containsKey("dataStatus") && StringUtils.isNotBlank(ObjectUtils.toString(map.get("dataStatus")))) {
            map.put("dataStatusArrs", Lists.newArrayList(map.get("dataStatus")));
        } else {
            map.put("dataStatusArrs", Lists.newArrayList(2,3,4));
        }

        return gyDataDao.examineList(map);
    }

    /**
     * @param file
     * @Description: 文件解密
     * @Author: wangp
     * @Date: 2022/3/9 15:07
     * @Return: boolean
     */
    @Override
    public String dec(MultipartFile file) {
        String decFile = "";
        FileInputStream srcStream = null;
        try {
            String fileName = file.getOriginalFilename();
            // 生成临时文件
            String fileNames = FileUtil.renameToUUID(fileName);
            String temFile = bootdoConfig.getUploadPath() + fileNames;
            MultipartFileToFile(file, temFile);
            // FileUtil.uploadFile(file.getBytes(), bootdoConfig.getUploadPath(), fileName);
            // 解密文件路径
            decFile = bootdoConfig.getUploadPath() + fileName;
            log.info("temFile:" + temFile + ",decFile:" + decFile);
            srcStream = new FileInputStream(temFile);
            log.info("srcStream:" + srcStream.toString());
            log.info("EncPrival:" + bootdoConfig.getEncPrival());
            FileCryptTool.decryptFile(bootdoConfig.getEncPrival(), temFile, decFile);
            // 清除数据
            srcStream.close();
            FileUtil.deleteFile(temFile);// 临时文件
        } catch (Exception e) {
            log.error("文件解密异常", e);
            return "";
        }
        return decFile;
    }

    /**
     * @description:  密文导入
     * @param:
     * @return: int
     * @author xhl
     * @date: 2022/9/7 14:28
     */
    @Override
    public int save(String decFlg,int type) {
        File file = new File(decFlg);
        InputStream is = null;
        int otherProvince = 0;
        try {
            is = new FileInputStream(file);
            XSSFWorkbook xssfWorkbook = new XSSFWorkbook(is);
            XSSFSheet xssfSheet = xssfWorkbook.getSheetAt(0);
            // 企业信息
            List<GyUnitDO> gyUnitList = new ArrayList();
            // 数据目录
            List<GyDataDO> dataList = new ArrayList();
            // 用户信息
            UserDO currUser = ShiroUtils.getUser();
            log.info("currUser:" + currUser.toString());
            String user = "-1";
            Long userId = null;
            String userProvince = null;
            Long deptId = Long.valueOf(-1);
            if (null != currUser) {
                userId = currUser.getUserId();
                user = currUser.getUsername();
                deptId = currUser.getDeptId();
                userProvince = currUser.getProvince();
            }
            // 遍历
            for (int i = 1; i <= xssfSheet.getLastRowNum(); i++) {
                log.info("userProvince: {}, i: {}", userProvince, i);

                Row row = xssfSheet.getRow(i);
                if (row.getCell(1) == null) {
                    break;
                }

                //所属省份
                String dataProvince = row.getCell(18).getStringCellValue();
                // 当非管理员导入的数据为非本省的数据的时候  跳过当前数据  并记录
                if (!managerValue.contains(deptId.toString())) {
                    log.info("地方主管部门");
                    if (!dataProvince.equals(userProvince)) {
                        log.info("当前用户省份: {}, 当前数据省份: {}, 数据名称: {}", userProvince, dataProvince, row.getCell(1).getStringCellValue());
                        otherProvince += 1;
                        continue;
                    }
                } else {
                    log.info("工信部 管理员");
                }

                // 企业单位
                GyUnitDO gyUnitDO = new GyUnitDO();
                // 数据信息
                GyDataDO gyDataDO = new GyDataDO();

                //无数据导出
                if ("/".equals(row.getCell(1).getStringCellValue()) && row.getCell(16).getStringCellValue() !=null && !"/".equals(row.getCell(16).getStringCellValue())) {
                    gyUnitDO.setReporterUnit(row.getCell(16).getStringCellValue());// 数据处理者名称
                    gyUnitDO.setCreditCode(row.getCell(17).getStringCellValue());// 统一社会信用代码(机构代码 )
                    gyUnitDO.setUnitProvince(dataProvince);//所在地区（省）
                    gyUnitDO.setUnitCity(row.getCell(19).getStringCellValue());//所在地区（市）
                    gyUnitDO.setUnitCharacter(row.getCell(20).getStringCellValue());//数据处理者性质
                    gyUnitDO.setIndustryTypeOne(row.getCell(21).getStringCellValue());//所属行业
                    gyUnitDO.setIndustryTypeTwo(row.getCell(22).getStringCellValue());//主营业务范围
                    gyUnitDO.setDataSecurityMan(row.getCell(23).getStringCellValue());//数据安全负责人姓名
                    gyUnitDO.setDataSecurityOffice(row.getCell(24).getStringCellValue());//数据安全负责人职务
                    gyUnitDO.setDataSecurityWay(row.getCell(25).getStringCellValue());//数据安全负责人联系方式
                    int count = gyUnitDao.getGyUnit(gyUnitDO);
                    if (count == 0) {
                        try {
                            gyUnitDO.setCreateTime(new Date());
                            gyUnitDO.setStatus("0");
                            gyUnitDao.save(gyUnitDO);
                        } catch (Exception e) {
                            log.error("error enterprise-only : {}", e.getMessage());
                        }
                    }
                    break;
                }

                gyDataDO.setDataName(row.getCell(1).getStringCellValue());//数据名称
                gyDataDO.setDataTypeBase(row.getCell(2).getStringCellValue());//数据分类分级规范
//                gyDataDO.setDataTypeRoot(row.getCell(3).getStringCellValue());//数据根类别
                gyDataDO.setDataTypeOne(row.getCell(3).getStringCellValue());//数据一级类别
                gyDataDO.setDataTypeTwo(row.getCell(4).getStringCellValue());//数据二级类别
                gyDataDO.setDataTypeThree(row.getCell(5)==null?null:row.getCell(5).getStringCellValue());//数据三级类别
                gyDataDO.setDataTypeFour(row.getCell(6)==null?null:row.getCell(6).getStringCellValue());//数据四级类别
                gyDataDO.setDataLevel(row.getCell(7).getStringCellValue());//数据级别
                gyDataDO.setDataCarrier(row.getCell(8).getStringCellValue());//数据载体
                gyDataDO.setDataSource(row.getCell(9).getStringCellValue());//数据来源
                gyDataDO.setDataNumGb(row.getCell(10).getNumericCellValue());//数据数量(非电子数据)
                gyDataDO.setDataNum((int) row.getCell(11).getNumericCellValue());//数据数量(电子数据)
                gyDataDO.setDataScope(row.getCell(12).getStringCellValue());//覆盖类型
                gyDataDO.setDataProportion(row.getCell(13).getStringCellValue());//覆盖占比
                gyDataDO.setDataPrecision(row.getCell(14).getStringCellValue());//数据精度
                gyDataDO.setDataPrecisionDes(row.getCell(15).getStringCellValue());//数据精度描述

                gyUnitDO.setReporterUnit(row.getCell(16).getStringCellValue());// 数据处理者名称
                gyUnitDO.setCreditCode(row.getCell(17).getStringCellValue());// 统一社会信用代码(机构代码 )
                gyUnitDO.setUnitProvince(dataProvince);//所在地区（省）
                gyUnitDO.setUnitCity(row.getCell(19).getStringCellValue());//所在地区（市）
//                gyUnitDO.setSupervisoryOrg(row.getCell(20).getStringCellValue());//所属监管部门
                gyUnitDO.setUnitCharacter(row.getCell(20).getStringCellValue());//数据处理者性质
                gyUnitDO.setIndustryTypeOne(row.getCell(21).getStringCellValue());//所属行业
                gyUnitDO.setIndustryTypeTwo(row.getCell(22).getStringCellValue());//主营业务范围
//                gyUnitDO.setUnitMan(row.getCell(23).getStringCellValue());//组织负责人姓名
//                gyUnitDO.setUnitOffice(row.getCell(24).getStringCellValue());//组织负责人职务
                gyUnitDO.setDataSecurityMan(row.getCell(23).getStringCellValue());//数据安全负责人姓名
//                gyUnitDO.setDataSecurityId(row.getCell(10).getStringCellValue());//数据安全负责人身份证号
                gyUnitDO.setDataSecurityOffice(row.getCell(24).getStringCellValue());//数据安全负责人职务
                gyUnitDO.setDataSecurityWay(row.getCell(25).getStringCellValue());//数据安全负责人联系方式
                gyUnitList.add(gyUnitDO);
                gyDataDO.setDataHandleType(row.getCell(26).getStringCellValue());//数据处理方式
                gyDataDO.setDataResult(row.getCell(27).getStringCellValue());//数据目的
                gyDataDO.setIsAlgorithmHandle(row.getCell(28).getStringCellValue());//是否涉及算法处理
                gyDataDO.setIsCross(row.getCell(29).getStringCellValue());//是否出境
                gyDataDO.setCrossAcceptName(row.getCell(30).getStringCellValue());//跨境接收方名称
//                gyDataDO.setDataCrossResult(row.getCell(34).getStringCellValue());//跨境目的
                gyDataDO.setDataCrossType(row.getCell(31).getStringCellValue());//出境方式
//                gyDataDO.setIsCrossEncrypt(row.getCell(31).getStringCellValue());//出境是否加密
                gyDataDO.setIsSecurityAss(row.getCell(32).getStringCellValue());//是否安全评估
                gyDataDO.setAssessResult(row.getCell(33).getStringCellValue());//评估结果
//                gyDataDO.setReorganize(row.getCell(34).getStringCellValue());//整改措施
//                gyDataDO.setIsApproval(row.getCell(35).getStringCellValue());//是否经网信部门审批
                gyDataDO.setIsCrossMain(row.getCell(34).getStringCellValue());//是否对外共享
                gyDataDO.setCrossmainAcceptName(row.getCell(35).getStringCellValue());//数据对外共享接收方
//                gyDataDO.setDataCrossmainResult(row.getCell(40).getStringCellValue());//数据对外共享目的
                gyDataDO.setDataCrossmainType(row.getCell(36).getStringCellValue());//数据对外共享方式
//                gyDataDO.setIsCrossmainEncrypt(row.getCell(40).getStringCellValue());//跨主体流动加密情况
//                gyDataDO.setCrossmainNum(row.getCell(41).getStringCellValue());//过去一年跨主体流动数量（单位：GB）
                gyDataDO.setIsForeignData(row.getCell(37).getStringCellValue());//是否为涉外数据
                gyDataDO.setIsCrossmainFlow(row.getCell(38).getStringCellValue());//是否涉及跨主体流动
                gyDataDO.setInfoSystemName(row.getCell(39).getStringCellValue());//信息系统名称
                if(row.getLastCellNum()>51){
                    gyDataDO.setInfoSystemIpAddress(row.getCell(40).getStringCellValue());//信息系统ip地址
                    gyDataDO.setInfoSystemDomainName(row.getCell(41).getStringCellValue());//信息系统域名
                    gyDataDO.setInfoSystemType(row.getCell(42).getStringCellValue());//信息系统类型
//                gyDataDO.setInfoSystemDes(row.getCell(43).getStringCellValue());//信息系统主要功能描述
//                gyDataDO.setInfoSystemEncrypt(row.getCell(44).getStringCellValue());//数据加密存储情况
                    gyDataDO.setSecurityCognizance(row.getCell(43).getStringCellValue());//网络安全等级保护认定情况
                    gyDataDO.setComSecurityCognizance(row.getCell(44).getStringCellValue());//通信网络安全防护定级备案情况
                    gyDataDO.setIsKetSystem(row.getCell(45).getStringCellValue());//是否为关键信息基础设施
                    gyDataDO.setIsKeydataAss(row.getCell(46).getStringCellValue());//是否开展重要数据安全风险评估
                    gyDataDO.setAssessOrg(row.getCell(47).getStringCellValue());//评估机构
                    gyDataDO.setAssessBase(row.getCell(48).getStringCellValue());//评估依据的规范
                    gyDataDO.setAssessTime(row.getCell(49).getStringCellValue());//评估时间
                    gyDataDO.setDataAssessResult(row.getCell(50).getStringCellValue());//重要数据安全风险评估结论
//                gyDataDO.setSecurityReorganize(row.getCell(52).getStringCellValue());//整改措施及整改情况
                    gyDataDO.setDataDesc(row.getCell(51).getStringCellValue());//备注
                } else {
                    gyDataDO.setInfoSystemType(row.getCell(40).getStringCellValue());//信息系统类型
                    gyDataDO.setSecurityCognizance(row.getCell(41).getStringCellValue());//网络安全等级保护认定情况
                    gyDataDO.setComSecurityCognizance(row.getCell(42).getStringCellValue());//通信网络安全防护定级备案情况
                    gyDataDO.setIsKetSystem(row.getCell(43).getStringCellValue());//是否为关键信息基础设施
                    gyDataDO.setIsKeydataAss(row.getCell(44).getStringCellValue());//是否开展重要数据安全风险评估
                    gyDataDO.setAssessOrg(row.getCell(45).getStringCellValue());//评估机构
                    gyDataDO.setAssessBase(row.getCell(46).getStringCellValue());//评估依据的规范
                    gyDataDO.setAssessTime(row.getCell(47).getStringCellValue());//评估时间
                    gyDataDO.setDataAssessResult(row.getCell(48).getStringCellValue());//重要数据安全风险评估结论
                    gyDataDO.setDataDesc(row.getCell(49).getStringCellValue());//备注
                    gyDataDO.setInfoSystemIpAddress("/");//信息系统ip地址
                    gyDataDO.setInfoSystemDomainName("/");//信息系统域名
                }

                gyDataDO.setGyUnitId(gyUnitDO.getCreditCode());//关联填报企业
                if (type == 1) {
                    gyDataDO.setSendVerify("0");// 送审状态 0:未送审 1：已送审
                    gyDataDO.setDataStatus("0"); // 状态 0:待审核 1:删除 2:已上报 3:通过审核 4:驳回
                    gyDataDO.setDeleteStatus("0"); // 状态 0:正常 1:删除
                } else {
                    gyDataDO.setSendVerify("1");// 送审状态 0:未送审 1：已送审
                    gyDataDO.setDataStatus("3"); // 状态 0:待审核 1:删除 2:已上报 3:通过审核 4:驳回
                    gyDataDO.setDeleteStatus("0"); // 状态 0:正常 1:删除
                }
                gyDataDO.setUploadTime(new Date());
                log.info("dataDO:" + gyDataDO.toString());
                Map<String, Object> params = new HashMap<>();
                params.put("dataName", gyDataDO.getDataName());// 数据名称
                params.put("gyUnitId", gyDataDO.getGyUnitId());// 单位
                //判断数据是否已经上传过
                List<GyDataImportDTO> listInfo = gyDataDao.listId(params);
                if (listInfo !=null && listInfo.size()>0) {
                    // 删除数据
                    log.info("删除数据id：" + listInfo.get(0).getId());
                    gyDataDao.remove(listInfo.get(0).getId());
                }
                dataList.add(gyDataDO);

            }
            // 企业单位批量导入
            log.info("unitList size:" + gyUnitList.size());
            if (gyUnitList != null && gyUnitList.size() > 0) {
                for (GyUnitDO gyUnitDO : gyUnitList) {
                    log.info("unitList size:" + gyUnitList.size());

                    int count = gyUnitDao.getGyUnit(gyUnitDO);
                    if (count == 0) {
                        try {
                            gyUnitDO.setCreateTime(new Date());
                            gyUnitDO.setStatus("0");
                            gyUnitDao.save(gyUnitDO);
                        } catch (Exception e) {
                            log.error("error enterprise-only : {}", e.getMessage());
                        }
                    }

                }
            }
            // 数据批量导入
            if (dataList != null && dataList.size() > 0) {
                gyDataDao.batchSave(dataList);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
                // xssfWorkbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            // 清除临时数据
            FileUtil.deleteFile(decFlg);
        }

        return otherProvince;
    }

    @Override
    public GyDataImportDTO get(Long id) {
        return gyDataDao.get(id);
    }

    @Override
    public GyDataExcelDO getDataExcel(Long id) {
        return gyDataDao.getDataExcel(id);
    }

    @Override
    public GbDataExcelDO getGbDataExcel(Long id) {
        return gyDataDao.getGbDataExcel(id);
    }

    @Override
    public void updateStatus(GyDataImportDTO data) {
        int effect = gyDataDao.updateStatus(data);
        if ("2".equalsIgnoreCase(data.getDataStatus()) && effect > 0) {
            GyDataImportDTO gyDataImportDTO = get(data.getId());
            RoleDO roleDO = roleService.get(gyDataImportDTO.getRoleId());
            if (roleDO != null) {
                ExamineLogDTO dto = new ExamineLogDTO();
                dto.setUniqueKey(String.valueOf(data.getId()));
                dto.setDataType("DI_OPER");
                dto.setMessage("数据已送审至: " + roleDO.getRoleName());
                examineLogService.save(dto);
            }
        } else if ("3".equalsIgnoreCase(data.getDataStatus()) && effect > 0) {
            GyDataImportDTO gyDataImportDTO = get(data.getId());
            RoleDO roleDO = roleService.get(gyDataImportDTO.getRoleId());
            if (roleDO != null) {
                ExamineLogDTO dto = new ExamineLogDTO();
                dto.setUniqueKey(String.valueOf(data.getId()));
                dto.setDataType("DI_OPER");
                dto.setMessage("数据已被(" + roleDO.getRoleName() + ")审核通过");
                examineLogService.save(dto);
            }
        } else if ("4".equalsIgnoreCase(data.getDataStatus()) && effect > 0) {
            GyDataImportDTO gyDataImportDTO = get(data.getId());
            RoleDO roleDO = roleService.get(gyDataImportDTO.getRoleId());
            if (roleDO != null) {
                ExamineLogDTO dto = new ExamineLogDTO();
                dto.setUniqueKey(String.valueOf(data.getId()));
                dto.setDataType("DI_OPER");
                dto.setMessage("数据已被(" + roleDO.getRoleName() + ")审核驳回, 请核对");
                examineLogService.save(dto);
            }
        }
    }

    @Override
    public int countTotal(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            // map.put("deptId", String.valueOf(currUser.getDeptId()));
            // map.put("nextVerifyDept", String.valueOf(currUser.getDeptId()));
            map.put("unitCity", String.valueOf(currUser.getCity()));
        } else {
            log.info("工信部 管理员");
        }
//        if (currUser.getExamineRoleId() != null) {
//            map.put("roleId", String.valueOf(currUser.getExamineRoleId()));
//        }
        return gyDataDao.countTotal(map);
    }

    @Override
    public int countExamineTotal(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            // map.put("deptId", String.valueOf(currUser.getDeptId()));
            // map.put("nextVerifyDept", String.valueOf(currUser.getDeptId()));
            map.put("unitCity", String.valueOf(currUser.getCity()));
        } else {
            log.info("工信部 管理员");
        }
        if (currUser.getExamineRoleId() != null) {
            map.put("roleId", String.valueOf(currUser.getExamineRoleId()));
        }
        return gyDataDao.countTotal(map);
    }

    @Override
    public List<GyDataImportDTO> listUnion(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        log.info("managerValue:" + managerValue);
        //
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("deptId", String.valueOf(currUser.getDeptId()));
            map.put("nextVerifyDept", String.valueOf(currUser.getDeptId()));
            map.put("unitProvince", String.valueOf(currUser.getProvince()));
        } else {
            log.info("工信部 管理员");
        }
        return gyDataDao.list(map);
    }

    @Override
    public int countUnion(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("deptId", String.valueOf(currUser.getDeptId()));
            map.put("nextVerifyDept", String.valueOf(currUser.getDeptId()));
            map.put("unitProvince", String.valueOf(currUser.getProvince()));
        } else {
            log.info("工信部 管理员");
        }
        return gyDataDao.countTotal(map);
    }

    @Override
    public void batchRemove(Long[] ids) {
        gyDataDao.batchRemove(ids);
    }

    /**
     * @description:  明文导入
     * @param: file
     * @return: java.lang.String
     * @author xhl
     * @date: 2022/9/7 14:23
     */
    @Override
    public String plainTextImport(MultipartFile file,int type) {
        InputStream is = null;
        String result = "";
        try {
            is = file.getInputStream();
            XSSFWorkbook xssfWorkbook = new XSSFWorkbook(is);
            XSSFSheet xssfSheet = xssfWorkbook.getSheetAt(0);
            // 企业信息
            List<GyUnitDO> gyUnitList = new ArrayList();
            // 数据目录
            List<GyDataDO> dataList = new ArrayList();
            // 用户信息
            UserDO currUser = ShiroUtils.getUser();
            log.info("currUser:" + currUser.toString());
            String user = "-1";
            Long userId = null;
            String userProvince = null;
            Long deptId = Long.valueOf(-1);
            if (null != currUser) {
                userId = currUser.getUserId();
                user = currUser.getUsername();
                deptId = currUser.getDeptId();
                userProvince = currUser.getProvince();
            }
            // 遍历
            for (int i = 4; i <= xssfSheet.getLastRowNum(); i++) {
                log.info("userProvince: {}, i: {}", userProvince, i);

                Row row = xssfSheet.getRow(i);
                if (row.getCell(1) == null) {
                    break;
                }
                //所属省份
                String dataProvince = row.getCell(18).getStringCellValue();
                // 当非管理员导入的数据为非本省的数据的时候  跳过当前数据  并记录
                if (!managerValue.contains(deptId.toString())) {
                    log.info("地方主管部门");
                    if (!dataProvince.equals(userProvince)) {
                        log.info("当前用户省份: {}, 当前数据省份: {}, 数据名称: {}", userProvince, dataProvince, row.getCell(1).getStringCellValue());
                        continue;
                    }
                } else {
                    log.info("工信部 管理员");
                }

                // 企业单位
                GyUnitDO gyUnitDO = new GyUnitDO();
                // 数据信息
                GyDataDO gyDataDO = new GyDataDO();

                //无数据导出
                if ("/".equals(row.getCell(1).getStringCellValue()) && row.getCell(16).getStringCellValue() !=null && !"/".equals(row.getCell(16).getStringCellValue())) {
                    gyUnitDO.setReporterUnit(row.getCell(16).getStringCellValue());// 数据处理者名称
                    gyUnitDO.setCreditCode(row.getCell(17).getStringCellValue());// 统一社会信用代码(机构代码 )
                    gyUnitDO.setUnitProvince(dataProvince);//所在地区（省）
                    gyUnitDO.setUnitCity(row.getCell(19).getStringCellValue());//所在地区（市）
                    gyUnitDO.setUnitCharacter(row.getCell(20).getStringCellValue());//数据处理者性质
                    gyUnitDO.setIndustryTypeOne(row.getCell(21).getStringCellValue());//所属行业
                    gyUnitDO.setIndustryTypeTwo(row.getCell(22).getStringCellValue());//主营业务范围
                    gyUnitDO.setDataSecurityMan(row.getCell(23).getStringCellValue());//数据安全负责人姓名
                    gyUnitDO.setDataSecurityOffice(row.getCell(24).getStringCellValue());//数据安全负责人职务
                    gyUnitDO.setDataSecurityWay(row.getCell(25).getStringCellValue());//数据安全负责人联系方式
                    int count = gyUnitDao.getGyUnit(gyUnitDO);
                    if (count == 0) {
                        try {
                            gyUnitDO.setCreateTime(new Date());
                            gyUnitDO.setStatus("0");
                            gyUnitDao.save(gyUnitDO);
                        } catch (Exception e) {
                            log.error("error enterprise-only : {}", e.getMessage());
                        }
                    }
                    break;
                }

                gyDataDO.setDataName(row.getCell(1).getStringCellValue());//数据名称
                gyDataDO.setDataTypeBase(row.getCell(2).getStringCellValue());//数据分类分级规范
//                gyDataDO.setDataTypeRoot(row.getCell(3).getStringCellValue());//数据根类别
                gyDataDO.setDataTypeOne(row.getCell(3).getStringCellValue());//数据一级类别
                gyDataDO.setDataTypeTwo(row.getCell(4).getStringCellValue());//数据二级类别
                gyDataDO.setDataTypeThree(row.getCell(5)==null?null:row.getCell(5).getStringCellValue());//数据三级类别
                gyDataDO.setDataTypeFour(row.getCell(6)==null?null:row.getCell(6).getStringCellValue());//数据四级类别
                gyDataDO.setDataLevel(row.getCell(7).getStringCellValue());//数据级别
                gyDataDO.setDataCarrier(row.getCell(8).getStringCellValue());//数据载体
                gyDataDO.setDataSource(row.getCell(9).getStringCellValue());//数据来源
                gyDataDO.setDataNumGb(row.getCell(10).getNumericCellValue());//数据数量(电子数据)
                gyDataDO.setDataNum((int)row.getCell(11).getNumericCellValue());//数据数量(非电子数据)
                gyDataDO.setDataScope(row.getCell(12).getStringCellValue());//覆盖类型
                gyDataDO.setDataProportion(row.getCell(13).getStringCellValue());//覆盖占比
                gyDataDO.setDataPrecision(row.getCell(14).getStringCellValue());//数据精度
                gyDataDO.setDataPrecisionDes(row.getCell(15).getStringCellValue());//数据精度描述

                gyUnitDO.setReporterUnit(row.getCell(16).getStringCellValue());// 数据处理者名称
                gyUnitDO.setCreditCode(row.getCell(17).getStringCellValue());// 统一社会信用代码(机构代码 )
                gyUnitDO.setUnitProvince(dataProvince);//所在地区（省）
                gyUnitDO.setUnitCity(row.getCell(19).getStringCellValue());//所在地区（市）
//                gyUnitDO.setSupervisoryOrg(row.getCell(20).getStringCellValue());//所属监管部门
                gyUnitDO.setUnitCharacter(row.getCell(20).getStringCellValue());//数据处理者性质
                gyUnitDO.setIndustryTypeOne(row.getCell(21).getStringCellValue());//所属行业
                gyUnitDO.setIndustryTypeTwo(row.getCell(22).getStringCellValue());//主营业务范围
//                gyUnitDO.setUnitMan(row.getCell(23).getStringCellValue());//组织负责人姓名
//                gyUnitDO.setUnitOffice(row.getCell(24).getStringCellValue());//组织负责人职务
                gyUnitDO.setDataSecurityMan(row.getCell(23).getStringCellValue());//数据安全负责人姓名
//                gyUnitDO.setDataSecurityId(row.getCell(10).getStringCellValue());//数据安全负责人身份证号
                gyUnitDO.setDataSecurityOffice(row.getCell(24).getStringCellValue());//数据安全负责人职务
                gyUnitDO.setDataSecurityWay(row.getCell(25).getStringCellValue());//数据安全负责人联系方式
                gyUnitList.add(gyUnitDO);
                gyDataDO.setDataHandleType(row.getCell(26).getStringCellValue());//数据处理方式
                gyDataDO.setDataResult(row.getCell(27).getStringCellValue());//数据目的
                gyDataDO.setIsAlgorithmHandle(row.getCell(28).getStringCellValue());//是否涉及算法处理
                gyDataDO.setIsCross(row.getCell(29).getStringCellValue());//是否出境
                gyDataDO.setCrossAcceptName(row.getCell(30).getStringCellValue());//跨境接收方名称
//                gyDataDO.setDataCrossResult(row.getCell(34).getStringCellValue());//跨境目的
                gyDataDO.setDataCrossType(row.getCell(31).getStringCellValue());//出境方式
//                gyDataDO.setIsCrossEncrypt(row.getCell(31).getStringCellValue());//出境是否加密
                gyDataDO.setIsSecurityAss(row.getCell(32).getStringCellValue());//是否安全评估
                gyDataDO.setAssessResult(row.getCell(33).getStringCellValue());//评估结果
//                gyDataDO.setReorganize(row.getCell(34).getStringCellValue());//整改措施
//                gyDataDO.setIsApproval(row.getCell(35).getStringCellValue());//是否经网信部门审批
                gyDataDO.setIsCrossMain(row.getCell(34).getStringCellValue());//是否对外共享
                gyDataDO.setCrossmainAcceptName(row.getCell(35).getStringCellValue());//数据对外共享接收方
//                gyDataDO.setDataCrossmainResult(row.getCell(40).getStringCellValue());//数据对外共享目的
                gyDataDO.setDataCrossmainType(row.getCell(36).getStringCellValue());//数据对外共享方式
//                gyDataDO.setIsCrossmainEncrypt(row.getCell(40).getStringCellValue());//跨主体流动加密情况
//                gyDataDO.setCrossmainNum(row.getCell(41).getStringCellValue());//过去一年跨主体流动数量（单位：GB）
                gyDataDO.setIsCrossmainFlow(row.getCell(37).getStringCellValue());//是否涉及跨主体流动
                gyDataDO.setIsForeignData(row.getCell(38).getStringCellValue());//是否为涉外数据
                gyDataDO.setInfoSystemName(row.getCell(39).getStringCellValue());//信息系统名称
                if(row.getLastCellNum()>49){
                    gyDataDO.setInfoSystemIpAddress(row.getCell(40).getStringCellValue());//信息系统ip地址
                    gyDataDO.setInfoSystemDomainName(row.getCell(41).getStringCellValue());//信息系统域名
                    gyDataDO.setInfoSystemType(row.getCell(42).getStringCellValue());//信息系统类型
//                gyDataDO.setInfoSystemDes(row.getCell(43).getStringCellValue());//信息系统主要功能描述
//                gyDataDO.setInfoSystemEncrypt(row.getCell(44).getStringCellValue());//数据加密存储情况
                    gyDataDO.setSecurityCognizance(row.getCell(43).getStringCellValue());//网络安全等级保护认定情况
                    gyDataDO.setComSecurityCognizance(row.getCell(44).getStringCellValue());//通信网络安全防护定级备案情况
                    gyDataDO.setIsKetSystem(row.getCell(45).getStringCellValue());//是否为关键信息基础设施
                    gyDataDO.setIsKeydataAss(row.getCell(46).getStringCellValue());//是否开展重要数据安全风险评估
                    gyDataDO.setAssessOrg(row.getCell(47).getStringCellValue());//评估机构
                    gyDataDO.setAssessBase(row.getCell(48).getStringCellValue());//评估依据的规范
                    gyDataDO.setAssessTime(row.getCell(49).getStringCellValue());//评估时间
                    gyDataDO.setDataAssessResult(row.getCell(50).getStringCellValue());//重要数据安全风险评估结论
//                gyDataDO.setSecurityReorganize(row.getCell(52).getStringCellValue());//整改措施及整改情况
                    gyDataDO.setDataDesc(row.getCell(51).getStringCellValue());//备注
                } else {
                    gyDataDO.setInfoSystemType(row.getCell(40).getStringCellValue());//信息系统类型
                    gyDataDO.setSecurityCognizance(row.getCell(41).getStringCellValue());//网络安全等级保护认定情况
                    gyDataDO.setComSecurityCognizance(row.getCell(42).getStringCellValue());//通信网络安全防护定级备案情况
                    gyDataDO.setIsKetSystem(row.getCell(43).getStringCellValue());//是否为关键信息基础设施
                    gyDataDO.setIsKeydataAss(row.getCell(44).getStringCellValue());//是否开展重要数据安全风险评估
                    gyDataDO.setAssessOrg(row.getCell(45).getStringCellValue());//评估机构
                    gyDataDO.setAssessBase(row.getCell(46).getStringCellValue());//评估依据的规范
                    gyDataDO.setAssessTime(row.getCell(47).getStringCellValue());//评估时间
                    gyDataDO.setDataAssessResult(row.getCell(48).getStringCellValue());//重要数据安全风险评估结论
                    gyDataDO.setDataDesc(row.getCell(49).getStringCellValue());//备注
                    gyDataDO.setInfoSystemIpAddress("/");//信息系统ip地址
                    gyDataDO.setInfoSystemDomainName("/");//信息系统域名
                }
                gyDataDO.setGyUnitId(gyUnitDO.getCreditCode());//关联填报企业
                if (type == 1) {
                    gyDataDO.setSendVerify("0");// 送审状态 0:未送审 1：已送审
                    gyDataDO.setDataStatus("0"); // 状态 0:待审核 1:删除 2:已上报 3:通过审核 4:驳回
                    gyDataDO.setDeleteStatus("0"); // 状态 0:正常 1:删除
                } else {
                    gyDataDO.setSendVerify("1");// 送审状态 0:未送审 1：已送审
                    gyDataDO.setDataStatus("3"); // 状态 0:待审核 1:删除 2:已上报 3:通过审核 4:驳回
                    gyDataDO.setDeleteStatus("0"); // 状态 0:正常 1:删除
                }
                gyDataDO.setUploadTime(new Date());
                log.info("dataDO:" + gyDataDO.toString());
                Map<String, Object> params = new HashMap<>();
                params.put("dataName", gyDataDO.getDataName());// 数据名称
                params.put("gyUnitId", gyDataDO.getGyUnitId());// 单位
                //判断数据是否已经上传过
                List<GyDataImportDTO> listInfo = gyDataDao.listId(params);
                if (listInfo !=null && listInfo.size()>0) {
                    // 删除数据
                    log.info("删除数据id：" + listInfo.get(0).getId());
                    gyDataDao.remove(listInfo.get(0).getId());
                }
                dataList.add(gyDataDO);

            }
            // 企业单位批量导入
            log.info("unitList size:" + gyUnitList.size());
            if (gyUnitList != null && gyUnitList.size() > 0) {
                for (GyUnitDO gyUnitDO : gyUnitList) {
                    log.info("unitList size:" + gyUnitList.size());

                    int count = gyUnitDao.getGyUnit(gyUnitDO);
                    if (count == 0) {
                        try {
                            gyUnitDO.setCreateTime(new Date());
                            gyUnitDO.setStatus("0");
                            gyUnitDao.save(gyUnitDO);
                        } catch (Exception e) {
                            log.error("error enterprise-only : {}", e.getMessage());
                        }
                    }

                }
            }
            // 数据批量导入
            if (dataList != null && dataList.size() > 0) {
                gyDataDao.batchSave(dataList);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
                // xssfWorkbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return result;
    }

    @Override
    public List<GyDataImportDTO> listRecover(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        log.info("managerValue:" + managerValue);
        //
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("unitProvince", String.valueOf(currUser.getProvince()));
        } else {
            log.info("工信部 管理员");
        }
        return gyDataDao.listRecover(map);
    }

    @Override
    public int countRecover(Map<String, Object> map) {
        UserDO currUser = ShiroUtils.getUser();
        log.info("currUser:" + currUser.toString());
        log.info("managerValue:" + managerValue);
        //
        if (!managerValue.contains(currUser.getDeptId().toString())) {
            log.info("地方主管部门");
            map.put("unitProvince", String.valueOf(currUser.getProvince()));
        } else {
            log.info("工信部 管理员");
        }
        return gyDataDao.countRecover(map);
    }

    @Override
    public int recover(Long id) {
        GyDataImportDTO gyDataImportDTO = gyDataDao.getRecoverUnit(id);
        if ("1".equals(gyDataImportDTO.getStatus())) {
            unitService.updateUnitByCreditCode(gyDataImportDTO.getGyUnitId());
        }
        Map<String, Object> params = new HashMap<>();
        params.put("dataName", gyDataImportDTO.getDataName());// 数据名称
        params.put("gyUnitId", gyDataImportDTO.getGyUnitId());// 单位
        List<GyDataImportDTO> listInfo = gyDataDao.list(params);
        if (listInfo !=null && listInfo.size()>0) {
            // 物理删除数据
            if (!CollectionUtils.isEmpty(listInfo)) {
                List<Long> collect = listInfo.stream().map(GyDataImportDTO::getId).collect(Collectors.toList());
                Long[] ids = collect.toArray(new Long[listInfo.size()]);
                gyDataDao.batchPhysicalDeletion(ids);
            }
        }
        return gyDataDao.recover(id);
    }

    @Override
    public int batchPhysicalDeletion(Long id) {

        Long[] longArray = new Long[]{id};
        return gyDataDao.batchPhysicalDeletion(longArray);
    }

    @Override
    public void encryptExports(HttpServletResponse responses, String idValue) throws Exception {
        List<GyDataExcelDO> list = new ArrayList<>();
        JSONArray json = (JSONArray)JSONArray.parse(idValue);
        for (int i = 0; i < json.size(); i++) {
            String id = String.valueOf(json.get(i));
            log.info("批量导出 id:" + json.get(i));
            GyDataExcelDO data = new GyDataExcelDO();
            data = gyDataDao.getDataExcel(Long.parseLong(id));
            data.setNo(i + 1); //序号
            list.add(data);
        }
        log.info("list size:" + list.size());
        String decFile = bootdoConfig.getUploadPath() + "数据目录备案导出信息.xlsx";
        String encFile = bootdoConfig.getUploadPath() + "数据目录备案加密导出信息.xlsx";

        ExcelUtils.encryptExport(responses, EncryptExcelDO.class,list,bootdoConfig.getCrtPrival(),decFile,encFile);
    }

    /**
     * @Description: MultipartFile转file
     * @Author: wangp
     * @Date: 2022/2/25 9:15
     * @param multiFile
     * @param fileName
     * @Return: File
     */
    public static File MultipartFileToFile(MultipartFile multiFile, String fileName) {
        // 获取文件后缀
        String prefix = fileName.substring(fileName.lastIndexOf("."));
        log.info("fileName:" + fileName + ",prefix:" + prefix);
        // 若须要防止生成的临时文件重复,能够在文件名后添加随机码
        try {
            File file = new File(fileName);
            boolean res = file.createNewFile();
            if (!res) {
                log.info("创建失败！");
            } else {
                multiFile.transferTo(file);
                file.setExecutable(true);// 设置可执行权限
                file.setReadable(true);// 设置可读权限
                file.setWritable(true);// 设置可写权限
            }
            return file;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String[] args) {
        FileCryptTool.encryptFile("F:/dev-dys/sm2/sm2_client_enc_cert.crt", "F:/dev-dys/res/测试导入-河南.xlsx",
                "F:/dev-dys/res/enc-测试导入加密文件-河南.xlsx");
        FileCryptTool.encryptFile("F:/dev-dys/sm2/sm2_client_enc_cert.crt", "F:/dev-dys/res/测试导入-河北.xlsx",
                "F:/dev-dys/res/enc-测试导入加密文件-河北.xlsx");
        FileCryptTool.encryptFile("F:/dev-dys/sm2/sm2_client_enc_cert.crt", "F:/dev-dys/res/测试导入-四川.xlsx",
                "F:/dev-dys/res/enc-测试导入加密文件-四川.xlsx");
    }
}
